import deampy.data_structure as df
import deampy.random_variats as RVGs

# last column includes the rate (e.g. mortality rate) of
# exponential distributions that should be created for each group.
rows = [
    # age, sex, race, mortality rate
    [0,     0,  0,  10],
    [0,     0,  1,  11],
    [0,     1,  0,  20],
    [0,     1,  1,  21],
    [5,     0,  0,  30],
    [5,     0,  1,  31],
    [5,     1,  0,  40],
    [5,     1,  1,  41]
]

df2 = df.DataFrameWithExpDist(rows=rows,
                              list_x_min=[0, 0, 0],
                              list_x_max=[5, 1, 1],
                              list_x_delta=[5, 'int', 'int'])

rng = RVGs.RNG(seed=1)
print('Sample 1:', df2.get_dist(x_value=[0, 0, 0]).sample(rng))
print('Sample 2', df2.get_dist(x_value=[1, 0, 0]).sample(rng))
print('Sample 3', df2.get_dist(x_value=[2, 1, 0]).sample(rng))
print('Sample 4', df2.get_dist(x_value=[5, 0, 1]).sample(rng))



from deampy import econ_eval as EV


S0 = EV.Strategy(name='Base', cost_obs=[100], effect_obs=[1])
S1 = EV.Strategy(name='A1', cost_obs=[800], effect_obs=[0.5])
S2 = EV.Strategy(name='A2', cost_obs=[2000], effect_obs=[10])
S3 = EV.Strategy(name='A3', cost_obs=[500], effect_obs=[7])
S4 = EV.Strategy(name='A4', cost_obs=[-100], effect_obs=[2])
S5 = EV.Strategy(name='A5', cost_obs=[200], effect_obs=[2])
S6 = EV.Strategy(name='A6', cost_obs=[1000], effect_obs=[7.2])
S7 = EV.Strategy(name='A7', cost_obs=[1100], effect_obs=[7.3])

cea = EV.CEA(strategies=[S0, S1, S2, S3, S4, S5, S6, S7], if_paired=False, health_measure='u')

# show the ce plane:
cea.plot_CE_plane()

# build table
cea.build_CE_table(interval_type='n')

print('On frontier')
frontier_strategies = cea.get_strategies_on_frontier()
for i, s in enumerate(frontier_strategies):
    print(s.name)
    if i > 0:
        print('incCost:', s.incCost.get_mean())
        print('incEffect:', s.incEffect.get_mean())
        print('ICER:', s.icer.get_ICER())



import deampy.markov as Cls

probMatrix = [
    [0.721,	0.202,	0.067,	0.010],
    [0.000,	0.581,	0.407,	0.012],
    [0.000,	0.000,	0.750,	0.250],
    [0.000,	0.000,	0.000,	1.000]
]

# discrete to continuous
transRateMatrix = Cls.discrete_to_continuous(
    trans_prob_matrix=probMatrix, delta_t=1)
print(transRateMatrix, '\n')

# continuous to discrete
newProbMatrix, prob2events = \
    Cls.continuous_to_discrete(trans_rate_matrix=transRateMatrix, delta_t=1)
print(newProbMatrix)
print('Upper bound for the probability of 2 transitions withing delta_t:', prob2events)





import numpy

from deampy import FormatFunctions as Support
from deampy import Statistics as Stat


def print_results(stat):
    print('   Average =', Support.format_number(stat.get_mean(), deci=3))
    print('   St Dev =', Support.format_number(stat.get_stdev(), deci=3))
    print('   Min =', Support.format_number(stat.get_min(), deci=3))
    print('   Max =', Support.format_number(stat.get_max(), deci=3))
    print('   Median =', Support.format_number(stat.get_percentile(50), deci=3))
    print('   95% Mean Confidence Interval (t-based) =',
          Support.format_interval(stat.get_t_CI(0.05), 3))
    print('   95% Mean Confidence Interval (bootstrap) =',
          Support.format_interval(stat.get_bootstrap_CI(0.05, 1000), 3))
    print('   95% Percentile Interval =',
          Support.format_interval(stat.get_PI(0.05), 3))


def mytest_summary_stat(data):
    # define a summary statistics
    sum_stat = Stat.SummaryStat(data=data, name='Test summary statistics',)
    print('Testing summary statistics:')
    print_results(sum_stat)


def mytest_discrete_time(data):
    # define a discrete-time statistics
    discrete_stat = Stat.DiscreteTimeStat('Test discrete-time statistics')
    # record data points
    for point in data:
        discrete_stat.record(point)

    print('Testing discrete-time statistics:')
    print_results(discrete_stat)


def mytest_continuous_time(times, observations):
    # define a continuous-time statistics
    continuous_stat = Stat.ContinuousTimeStat(initial_time=0, name='Test continuous-time statistics')

    for obs in range(0, len(times)):
        # find the increment
        inc = 0
        if obs == 0:
            inc = observations[obs]
        else:
            inc = observations[obs] - observations[obs - 1]
        continuous_stat.record(times[obs], inc)

    print('Testing continuous-time statistics:')
    print_results(continuous_stat)


def mytest_diff_stat_indp(x, y):
    # define
    stat = Stat.DifferenceStatIndp(x, y, name='Test DifferenceStatIndp')
    print('Testing DifferenceStatIndp:')
    print_results(stat)


def mytest_diff_stat_paired(x, y):
    # define
    stat = Stat.DifferenceStatPaired(x, y, name='Test DifferenceStatPaired')
    print('Testing DifferenceStatPaired:')
    print_results(stat)


def mytest_ratio_stat_indp(x, y):
    # define
    stat = Stat.RatioStatIndp(x, y, name='Test RatioStatIndp')
    print('Testing RatioStatIndp:')
    print_results(stat)


def mytest_ratio_stat_paied(x, y):
    # define
    stat = Stat.RatioStatPaired(x, y, name='Test RatioStatPaired')

    print('Testing RatioStatPaired:')
    print_results(stat)


def mytest_relativeDiff_stat_paied(x, y):
    # define
    stat = Stat.RelativeDifferencePaired(x, y, name='Test RelativeDifferencePaired')

    print('Testing RelativeDifferencePaired:')
    print_results(stat)

def mytest_relativeDiff_stat_indp(x, y):
    # define
    stat = Stat.RelativeDifferenceIndp(x, y, name='Test RelativeDifferenceIndp')

    print('Testing RelativeDifferenceIndp:')
    print_results(stat)


# generate sample data
x = numpy.random.normal(10, 4, 1000)
y_ind = numpy.random.normal(5, 2, 1000)
delta = numpy.random.normal(5, 1, 1000)
y_diff_paired = x - delta
ratio = numpy.random.normal(2, 1, 1000)
y_ratio_paired = numpy.divide(x, ratio)

relative_ratio = numpy.random.normal(0.5, 0.1, 1000)
y_relativeRatio_paired = numpy.divide(x, 1+relative_ratio)


# populate a data set to test continuous-time statistics
sampleT = []
sampleObs = []
i = 0
for i in range(0, 100):
    t = numpy.random.uniform(i, i + 1)
    sampleT.append(t)
    sampleObs.append(10*t)

# test summary statistics
mytest_summary_stat(x)
# test discrete-time statistics
mytest_discrete_time(x)
# test continuous-time statistics
mytest_continuous_time(sampleT, sampleObs)
# test statistics for the difference of two independent samples
mytest_diff_stat_indp(x, y_ind)
# test statistics for the difference of two paired samples
mytest_diff_stat_paired(x, y_diff_paired)
# test statistics for the ratio of two independent samples
mytest_ratio_stat_indp(x, y_ind)
# test statistics for the ratio of two paired samples
mytest_ratio_stat_paied(x, y_ratio_paired)
# test statistics for the relative difference of two paired samples
mytest_relativeDiff_stat_paied(x, y_relativeRatio_paired)
# test statistics for the relative difference of two independent samples
mytest_relativeDiff_stat_indp(x, y_ind)


import deampy.plots.sample_paths as Path

# a sample path with initial size = 1
path1 = Path.IncidenceSamplePath('Path 1', delta_t=1, sim_rep=1)
# record the observations
path1.record_increment(1.2, 2)
path1.record_increment(2.3, 1)
path1.record_increment(5.1, 5)
path1.close(6)
# stats
print(path1.stat.get_mean())

# second sample path with initial size = 1
path2 = Path.IncidenceSamplePath('Path 2', delta_t=1, sim_rep=1)
# record the observations
path2.record_increment(0.5, 4)
path2.record_increment(1.8, 2)
path2.record_increment(5.5, 1)
path2.close(6)
# stats
print(path2.stat.get_mean())

# plot path 1 only
Path.plot_sample_path(
    sample_path=path1,
    title='Plotting a single sample path',
    x_label='Time Period',
    y_label='observed value',
    legend='Path 1',
    color_code='r',
    connect='line')

Path.plot_sample_path(
    sample_path=path2,
    title='Plotting a single sample path',
    x_label='Time Period',
    y_label='observed value',
    legend='Path 2',
    color_code='r',
    connect='line')



import random

import matplotlib.pyplot as plt
import numpy as np

import deampy.plots.plot_support as Fig
import deampy.support.misc_functions as S


def plot_eff_sample_size(likelihood_weights, if_randomize=True,
                         fig_size=(6, 5), file_name=None,
                         title=None, x_label='Iteration', y_label='Effective Sample Size',
                         x_range=None, y_range=None
                         ):

    # convert the data to np array if needed
    if not type(likelihood_weights) == np.ndarray:
        likelihood_weights = np.array(likelihood_weights)

    # randomize the probabilities if needed
    if if_randomize:
        random.seed(1)
        random.shuffle(likelihood_weights)

    # calculate the effectiveve sample sizes through iterations
    effs = []
    for i in range(len(likelihood_weights)):
        effs.append(S.effective_sample_size(likelihood_weights[:i + 1]))

    # plot
    fig, ax = plt.subplots(figsize=fig_size)
    ax.plot(range(1, len(likelihood_weights) + 1), effs)

    ax.set_title(title)
    ax.set_xlabel(x_label)
    ax.set_ylabel(y_label)
    ax.set_xlim(x_range)
    ax.set_ylim(y_range)

    # if f_star is not None:
    #     plt.axhline(y=f_star, linestyle='--', color='black', linewidth=1)
    Fig.output_figure(fig, filename=file_name)




import numpy as np

import deampy.econ_eval as EconEval

np.random.seed(573)

cost_base = np.random.normal(loc=10000, scale=100, size=1000)
effect_base = np.random.normal(loc=1, scale=.1, size=1000)
cost_intervention = np.random.normal(loc=20000, scale=200, size=1000)
effect_intervention = np.random.normal(loc=2, scale=.2, size=1000)

print('')

# ICER calculation assuming paired observations
ICER_paired = EconEval.ICER_Paired(name='Testing paired ICER',
                                   costs_new=cost_intervention,
                                   effects_new=effect_intervention,
                                   costs_base=cost_base,
                                   effects_base=effect_base)
print('Paired ICER:'
      '\n\tICER: {} '
      '\n\tCI (boostrap): {} '
      '\n\tCI (Bayesian): {} '
      '\n\tPI: {}'.format(
      ICER_paired.get_ICER(),
      ICER_paired.get_CI(0.05),
      ICER_paired.get_CI(0.05, method='Bayesian'),
      ICER_paired.get_PI(0.05)))

# ICER calculation assuming independent observations
ICER_indp = EconEval.ICER_Indp('Testing independent ICER',
                               costs_new=cost_intervention,
                               effects_new=effect_intervention,
                               costs_base=cost_base,
                               effects_base=effect_base)
print('Independent ICER (confidence and prediction interval): ',
      ICER_indp.get_ICER(),
      ICER_indp.get_CI(0.05),
      ICER_indp.get_PI(0.05, ))

# NMB
NMB_paired = EconEval.INMB_Paired("Testing paired NMB",
                                  costs_new=cost_intervention,
                                  effects_new=effect_intervention,
                                  costs_base=cost_base,
                                  effects_base=effect_base)
print('Paired NMB (confidence and prediction interval): ',
      NMB_paired.get_INMB(wtp=10000),
      NMB_paired.get_CI(wtp=10000, alpha=.05),
      NMB_paired.get_PI(wtp=10000, alpha=.05))

NMB_indp = EconEval.INMB_Indp("Testing independent NMB",
                              costs_new=cost_intervention,
                              effects_new=effect_intervention,
                              costs_base=cost_base,
                              effects_base=effect_base)
print('Independent NMB (confidence and prediction interval): ',
      NMB_indp.get_INMB(wtp=10000),
      NMB_indp.get_CI(wtp=10000, alpha=.05),
      NMB_indp.get_PI(wtp=10000, alpha=.05))

print('')



import numpy as np

from deampy import econ_eval as ce

np.random.seed(573)
# create the centers of strategies
s_center = np.random.normal(0, 5000, (10, 2))

s0 = ce.Strategy("s1", s_center[0, 0]+np.random.normal(0, 200, 10), s_center[0, 1]+np.random.normal(0, 200, 10))
s1 = ce.Strategy("s2", s_center[1, 0]+np.random.normal(0, 200, 10), s_center[1, 1]+np.random.normal(0, 100, 10))
s2 = ce.Strategy("s3", s_center[2, 0]+np.random.normal(0, 200, 10), s_center[2, 1]+np.random.normal(0, 200, 10))
s3 = ce.Strategy("s4", s_center[3, 0]+np.random.normal(0, 200, 10), s_center[3, 1]+np.random.normal(0, 200, 10))
s4 = ce.Strategy("s5", s_center[4, 0]+np.random.normal(0, 200, 10), s_center[4, 1]+np.random.normal(0, 200, 10))
s5 = ce.Strategy("s6", s_center[5, 0]+np.random.normal(0, 200, 10), s_center[5, 1]+np.random.normal(0, 200, 10))
s6 = ce.Strategy("s7", s_center[6, 0]+np.random.normal(0, 200, 10), s_center[6, 1]+np.random.normal(0, 200, 10))
s7 = ce.Strategy("s8", s_center[7, 0]+np.random.normal(0, 200, 10), s_center[7, 1]+np.random.normal(0, 200, 10))
s8 = ce.Strategy("s9", s_center[8, 0]+np.random.normal(0, 200, 10), s_center[8, 1]+np.random.normal(0, 200, 10))
s9 = ce.Strategy("s10", s_center[9, 0]+np.random.normal(0, 200, 10), s_center[9, 1]+np.random.normal(0, 200, 10))

# create a CEA object -- unpaired
myCEA = ce.CEA([s0, s1, s2, s3, s4, s5, s6, s7, s8, s9], if_paired=False)

# plot with label and sample cloud
myCEA.plot_CE_plane('CE plane with unpaired observations and showing labels',
                    x_label='E[Effect]', y_label='E[Cost]', show_legend=True, add_clouds=True, fig_size=(6, 6))

# table
print('')
myCEA.build_CE_table(interval_type='c',
                     cost_digits=0, effect_digits=0, icer_digits=1,
                     file_name='Table-Indp.csv')

# plot with label and sample cloud
myCEA.plot_CE_plane('CE plane with paired observations and showing labels',
                    x_label='E[Effect]', y_label='E[Cost]', show_legend=True, add_clouds=True, fig_size=(6, 6))

dict = myCEA.get_dCost_dEffect_cer(interval_type='c', alpha=0.05,
                                   cost_digits=0, effect_digits=0, icer_digits=1)
print('dCost, dEffect, CER')
for s in dict:
    print(s, dict[s])

# create a CEA object -- paired
myCEA2 = ce.CEA([s0, s1, s2, s3, s4, s5, s6, s7, s8, s9], if_paired=True)


# frontier results
print('')
print('Strategies on the frontier:')
frontier = myCEA2.get_strategies_on_frontier()
for s in frontier:
    print('name:', s.name)
    print('cost', s.cost.get_mean())
    print('dCost', s.dCost.get_mean())
    if s.incCost is None:
        print('incCost', None)
    else:
        print('incCost', s.incCost.get_mean())








import math

import numpy as np
import scipy.stats as scipy

import deampy.random_variats as RVGs


def print_test_results(dist_name, samples, expectation, variance):
    print('Testing ' + dist_name + ':')
    print('  E[x] = {ex:.{prec}f} | Sample mean = {sm:.{prec}f}'.format(
        ex=expectation, sm=np.average(samples), prec=3))
    print('  Var[x] = {var:.{prec}f} | Sample variance = {sv:.{prec}f}\n'.format(
        var=variance, sv=np.var(samples), prec=3))


def print_test_results_multivariate(dist_name, samples, expectation, variance, axis):
    print('Testing ' + dist_name + ':')
    print("  E[x] = %(ex)s | Sample mean = %(sm)s" % {'ex':expectation, 'sm': np.average(samples, axis=axis)})
    print("  Var[x] = %(var)s | Sample variance = %(sv)s\n" % \
          {'var': variance, 'sv': np.var(samples, axis=axis)})


def get_samples(dist, rnd):
    """ sampling from a uni-variate distribution """

    samples = []
    for i in range(0, 10000):
        # get 10000 samples
        samples.append(dist.sample(rnd))
    return samples


def get_samples_multivariate(dist, rnd):
    """ sampling from a multi-variate distribution """

    samples = np.zeros([len(dist.a), 10000])
    for i in range(0, 10000):
        # get 10000 samples
        samples[:, i] = dist.sample(rnd)
    return samples


def test_rng(rnd):
    # obtain samples
    samples=[]
    for i in range(10000):
        samples.append(rnd.sample())
    
    # report mean and variance
    print_test_results('RNG', samples,
                       expectation=0.5,
                       variance=1/12)


def test_exponential(rnd, scale, loc=0):

    # exponential random variate generator
    exp_dist = RVGs.Exponential(scale, loc)

    # obtain samples
    samples = get_samples(exp_dist, rnd)

    # report mean and variance
    print_test_results('Exponential', samples,
                       expectation=scale+loc,
                       variance=scale**2)


def test_bernoulli(rnd, p):

    # bernoulli random variate generator
    bernoulli_dist = RVGs.Bernoulli(p)

    # obtain samples
    samples = get_samples(bernoulli_dist, rnd)

    # report mean and variance
    print_test_results('Bernoulli', samples,
                       expectation=p,
                       variance=p*(1-p))


def test_beta(rnd, a, b, loc=0, scale=1):

    # beta random variate generator
    beta_dist = RVGs.Beta(a, b, loc, scale)

    # obtain samples
    samples = get_samples(beta_dist, rnd)

    # report mean and variance
    print_test_results('Beta', samples,
                       expectation=(a*1.0/(a + b))*scale + loc,
                       variance=((a*b)/((a+b+1)*(a+b)**2.0)) * scale**2)


def test_beta_binomial(rnd, n, a, b, loc=0):

    # beta random variate generator
    beta_binomial_dist = RVGs.BetaBinomial(n, a, b, loc)

    # obtain samples
    samples = get_samples(beta_binomial_dist, rnd)

    # report mean and variance
    print_test_results('BetaBinomial', samples,
                       expectation=(a*n/(a + b)) + loc,
                       variance=(n*a*b*(a+b+n))/((a+b)**2*(a+b+1)))


def test_binomial(rnd, n, p, loc=0):

    # bimonial random variate generator
    binomial_dist = RVGs.Binomial(n, p, loc)

    # obtain samples
    samples = get_samples(binomial_dist, rnd)

    # report mean and variance
    print_test_results('Binomial', samples,
                       expectation=n*p + loc,
                       variance=n*p*(1-p))


def test_dirichlet(rnd, a):
    # dirichlet random variate generator
    dirichlet_dist = RVGs.Dirichlet(a)

    # obtain samples
    samples = get_samples_multivariate(dirichlet_dist, rnd)

    # report mean and variance
    a0 = sum(a)
    if type(a) == list:
        a = np.array(a)
    mean = a * (1.0/a0)
    var = np.zeros(len(a))
    for i in range(len(a)):
        var[i] = (a[i]*(a0-a[i]))/(((a0)**2)*(a0+1.0))

    print_test_results_multivariate('Dirichlet', samples,
                                    expectation=mean,
                                    variance=var,
                                    axis=1)


def test_empirical(rnd, prob):
    # empirical random variate generator
    empirical_dist = RVGs.Empirical(prob)

    # obtain samples
    samples = get_samples(empirical_dist, rnd)

    # report mean and variance
    if type(prob) == list:
        prob = np.array(prob)

    outcome = np.array(range(len(prob)))

    mean = sum(outcome*prob)
    var = sum((outcome**2)*prob) - mean**2

    print_test_results('Empirical', samples,
                       expectation=mean,
                       variance=var)


def test_gamma(rnd, a, loc=0, scale=1):
    # gamma random variate generator
    gamma_dist = RVGs.Gamma(a=a, scale=scale, loc=loc)

    # obtain samples
    samples = get_samples(gamma_dist, rnd)

    # report mean and variance
    print_test_results('Gamma', samples,
                       expectation=a*scale + loc,
                       variance=a*scale**2)


def test_gamma_poisson(rnd, a, gamma_scale, loc=0):
    # gamma-poisson random variate generator
    gamma_poisson_dist = RVGs.GammaPoisson(a=a, gamma_scale=gamma_scale, loc=loc)

    # obtain samples
    samples = get_samples(gamma_poisson_dist, rnd)

    # report mean and variance
    print_test_results('GammaPoisson', samples,
                       expectation=(a*gamma_scale) + loc,
                       variance=a*gamma_scale + a*(gamma_scale**2))


def test_geometric(rnd, p, loc=0):
    # geometric random variate generator
    geometric_dist = RVGs.Geometric(p, loc)

    # obtain samples
    samples = get_samples(geometric_dist, rnd)

    # report mean and variance
    print_test_results('Geometric', samples,
                       expectation=1/p + loc,
                       variance=(1-p)/(p**2))


def test_johnsonsb(rnd, a, b, loc, scale):
    # johnsonSb random variate generator
    johnsonsb_dist = RVGs.JohnsonSb(a, b, loc, scale)

    # obtain samples
    samples = get_samples(johnsonsb_dist, rnd)

    # report mean and variance
    mean = scipy.johnsonsb.mean(a,b,loc,scale)
    var = scipy.johnsonsb.var(a,b,loc,scale)

    print_test_results('JohnsonSb', samples,
                       expectation=mean,
                       variance=var)


def test_johnsonsu(rnd, a, b, loc, scale):
    # johnsonSu random variate generator
    johnsonsu_dist = RVGs.JohnsonSu(a, b, loc, scale)

    # obtain samples
    samples = get_samples(johnsonsu_dist, rnd)

    # report mean and variance
    mean = scipy.johnsonsu.mean(a,b,loc,scale)
    var = scipy.johnsonsu.var(a,b,loc,scale)

    print_test_results('JohnsonSu', samples,
                       expectation=mean,
                       variance=var)


def test_lognormal(rnd, mu, sigma, loc):
    #lognormal random variate generator
    lognormal_dist = RVGs.LogNormal(mu=mu, sigma=sigma,loc=loc)

    # obtain samples
    samples = get_samples(lognormal_dist, rnd)

    # report mean and variance
    print_test_results('LogNormal', samples,
                       expectation=np.exp(mu + 0.5*sigma**2) + loc,
                       variance=(np.exp(sigma**2) - 1.0) * np.exp(2*mu + sigma**2)
                       )


def test_multinomial(rnd, n, pvals):
    # multinomial random variate generator
    multinomial_dist = RVGs.Multinomial(n, pvals)

    # obtain samples
    samples = get_samples(multinomial_dist, rnd)

    pvals = np.array(pvals)

    # report mean and variance
    print_test_results_multivariate('Multinomial', samples,
                                    expectation=n*pvals,
                                    variance=n*pvals*(1-pvals),
                                    axis=0
                                    )


def test_negative_binomial(rnd, n, p, loc=0):

    # negative bimonial random variate generator
    negative_binomial_dist = RVGs.NegativeBinomial(n, p, loc)

    # obtain samples
    samples = get_samples(negative_binomial_dist, rnd)

    # get theoretical mean and variance
    mean = scipy.nbinom.stats(n, p, loc, moments='m')
    mean = np.asarray(mean).item()
    var = scipy.nbinom.stats(n, p, loc, moments='v')
    var = np.asarray(var).item()

    # report mean and variance
    print_test_results('Negative Binomial', samples,
                       expectation=mean,
                       variance=var
                       )


def test_non_homogeneous_exponential(rnd, rates, delta_t=1):
    # non homogeneous exponential random variate generator
    nhexp_dist = RVGs.NonHomogeneousExponential(rates=rates, delta_t=delta_t)

    # obtain samples
    samples = get_samples(nhexp_dist, rnd)

    # report mean and variance
    print_test_results('Non-Homogeneous Exponential', samples,
                       expectation=0,
                       variance=0
                       )


def test_normal(rnd, loc=0, scale=1):
    #normal random variate generator
    normal_dist = RVGs.Normal(loc, scale)

    # obtain samples
    samples = get_samples(normal_dist, rnd)

    # report mean and variance
    print_test_results('Normal', samples,
                       expectation=loc,
                       variance=scale**2
                       )


def test_poisson(rnd, mu, loc=0):
    # poisson random variate generator
    poisson_dist = RVGs.Poisson(mu, loc)

    # obtain samples
    samples = get_samples(poisson_dist, rnd)

    # report mean and variance
    print_test_results('Poisson', samples,
                       expectation=mu+loc,
                       variance=mu
                       )


def test_triangular(rnd, c, loc=0, scale=1):
    # triangular random variate generator
    triangular_dist = RVGs.Triangular(c, loc, scale)

    # obtain samples
    samples = get_samples(triangular_dist, rnd)

    # get theoretical variance
    var = scipy.triang.stats(c, loc, scale, moments='v')
    var = np.asarray(var).item()

    # report mean and variance
    print_test_results('Triangular', samples,
                       expectation=(3*loc + scale+ c*scale)/3.0,
                       variance=var
                       )


def test_uniform(rnd, loc=0, scale=1):
    # uniform random variate generator
    uniform_dist = RVGs.Uniform(scale=scale, loc=loc)

    # obtain samples
    samples = get_samples(uniform_dist, rnd)

    # report mean and variance
    print_test_results('Uniform', samples,
                       expectation=(2*loc + scale) / 2.0,
                       variance=scale**2/12.0
                       )


def test_uniform_discrete(rnd, l, u):
    # uniform discrete random variate generator
    uniformdiscrete_dist = RVGs.UniformDiscrete(l, u)

    # obtain samples
    samples = get_samples(uniformdiscrete_dist, rnd)

    # report mean and variance
    print_test_results('Uniform Discrete', samples,
                       expectation=(l + u) / 2.0,
                       variance=((u - l + 1) ** 2 - 1) / 12.0
                       )


def test_weibull(rnd, a, loc=0, scale=1):
    # weibull random variate generator
    weibull_dist = RVGs.Weibull(a=a, scale=scale,loc=loc)

    # obtain samples
    samples = get_samples(weibull_dist, rnd)

    # get theoretical variance
    var = scipy.weibull_min.stats(a, loc, scale, moments='v')
    var = np.asarray(var).item()

    # report mean and variance
    print_test_results('Weibull', samples,
                       expectation=math.gamma(1.0 + 1/a) * scale + loc,
                       variance=var
                       )



import compare as C
from deampy.optimization.appx_policy_itr import MultiApproximatePolicyIteration
from deampy.optimization.learning_exploring_rules import *
from model import Model

N_ITRS = 1000
IF_PARALLEL = False
B = [25, 50]
BETA = [0.4, 0.5]
Q_FUNC_DEGREES = [2]
L2_PENALTIES = [0.01]

if __name__ == '__main__':

    # build models
    n = len(B) * len(BETA) * len(Q_FUNC_DEGREES) * len(L2_PENALTIES)
    models = []
    for i in range(n):
        models.append(Model(cost_sigma=C.COST_SIGMA, action_cost=C.ACTION_COST))

    optimizer = MultiApproximatePolicyIteration(
        sim_models=models,
        num_of_actions=1,
        learning_rules=[Harmonic(b=b) for b in B],
        exploration_rules=[EpsilonGreedy(beta=beta) for beta in BETA],
        q_function_degrees=Q_FUNC_DEGREES,
        l2_penalties=L2_PENALTIES)

    optimizer.minimize_all(n_iterations=N_ITRS, n_last_itrs_to_find_minimum=int(N_ITRS*0.2),
                           if_parallel=IF_PARALLEL, folder_to_save_iterations='optimization_results')

    optimizer.plot_iterations(moving_ave_window=int(N_ITRS / 20), fig_size=(5, 6),
                              folder_to_save_figures='optimization_figures')

    C.compare(q_function_degree=Q_FUNC_DEGREES[0],
              q_functions_csvfile='optimal_q_functions.csv')




import numpy as np

import deampy.optimization.stoch_approx as Opt
from tests.optimization import toy_model

# create an object for the stochastic approximation method
mySimOpt = Opt.StochasticApproximation(
    sim_model=toy_model.Xto2(err_sigma=10),
    derivative_step=Opt.StepSize_Df(c0=5),
    step_size=Opt.StepSize_GeneralizedHarmonic(a0=5, b=100))

# find the minimum
mySimOpt.minimize(max_itr=5000, n_last_itrs_to_ave=200, x0=np.array([20, 5]))

# plot x and objective function values
mySimOpt.plot_f_itr(f_star=0)
mySimOpt.plot_x_irs(x_stars=[-1, 0])
mySimOpt.plot_Df_irs()
mySimOpt.plot_step_move()
mySimOpt.plot_step_Df()


print('Optimal x: ', mySimOpt.xStar)
print('Optimal f: ', mySimOpt.fStar)




import time


def fib(n):
    if n < 2:
        return n
    return fib(n-2) + fib(n-1)


def fib_cached(n, cache={}):
    if n < 2:
        return n
    try:
        val = cache[n]
    except KeyError:
        val = fib_cached(n-2, cache) + fib_cached(n-1, cache)
        cache[n] = val
    return val


N = 40
start = time.time()
v = fib(N)
end = time.time()
print(v, end - start)

start = time.time()
v = fib_cached(N)
end = time.time()
print(v, end - start)





def format_ax(ax,
              x_range=None, x_delta=None,
              y_range=None, y_delta=None, if_y_axis_prob=True,
              if_format_y_numbers=True, y_axis_decimal=1):

    # the range of x and y-axis are set so that we can get the
    # tick values and label
    if y_range is None and if_y_axis_prob:
        ax.set_ylim((-0.01, 1.01))
    if y_range:
        ax.set_ylim(y_range)
    if x_range:
        ax.set_xlim(x_range)

    # get x ticks
    if x_delta is None:
        vals_x = ax.get_xticks()
    else:
        vals_x = []
        x = x_range[0]
        while x <= x_range[1]:
            vals_x.append(x)
            x += x_delta

    # get y ticks
    if y_delta is None:
        vals_y = ax.get_yticks()
    else:
        vals_y = []
        y = y_range[0]
        while y <= y_range[1]:
            vals_y.append(y)
            y += y_delta

    # format x-axis
    ax.set_xticks(vals_x)
    ax.set_xticklabels(['{:,.{prec}f}'.format(x, prec=0) for x in vals_x])

    d = 2 * (x_range[1] - x_range[0]) / 200
    ax.set_xlim([x_range[0] - d, x_range[1] + d])

    # format y-axis
    if y_range is None:
        ax.set_yticks(vals_y)
    if if_y_axis_prob:
        ax.set_yticklabels(['{:.{prec}f}'.format(x, prec=1) for x in vals_y])
    elif if_format_y_numbers:
        ax.set_yticklabels(['{:,.{prec}f}'.format(x, prec=y_axis_decimal) for x in vals_y])

    if y_range is None and if_y_axis_prob:
        ax.set_ylim((-0.01, 1.01))
    if y_range:
        ax.set_ylim(y_range)

    if not if_y_axis_prob:
        ax.axhline(y=0, c='k', ls='--', linewidth=0.5)


def old_add_curves_to_ax(ax, curves, legends, x_range, x_delta, y_range, show_legend,
                         line_width, opt_line_width, legend_font_size,
                         y_axis_multiplier=1, y_axis_decimal=1,
                         opt='max', if_y_axis_prob=True):

    for i, curve in enumerate(curves):
        # plot line
        if legends is None:
            ax.plot(curve.xs, curve.ys*y_axis_multiplier, c=curve.color, alpha=1,
                    label=curve.label, linewidth=line_width)
        else:
            ax.plot(curve.xs, curve.ys*y_axis_multiplier, c=curve.color, alpha=1,
                    label=legends[i], linewidth=line_width)

        if opt == 'max':
            ax.plot(curve.maxXs, curve.maxYs*y_axis_multiplier, c=curve.color, linewidth=opt_line_width)
        elif opt == 'min':
            ax.plot(curve.minXs, curve.minYs*y_axis_multiplier, c=curve.color, linewidth=opt_line_width)
        else:
            raise ValueError('opt parameter should be min or max.')

    format_ax(ax=ax, y_range=y_range,
              x_range=x_range,
              x_delta=x_delta,
              if_y_axis_prob=if_y_axis_prob,
              if_format_y_numbers=True,
              y_axis_decimal=y_axis_decimal)

    if show_legend:
        ax.legend(fontsize=legend_font_size)  # xx-small, x-small, small, medium, large, x-large, xx-large


def add_curves_to_ax(ax, curves, title,
                     x_values, x_label, y_label, y_range=None,
                     y_axis_multiplier=1, y_axis_decimal=1,
                     x_delta=None,
                     transparency_lines=0.4,
                     transparency_intervals=0.2,
                     show_legend=False,
                     show_frontier=True,
                     show_labels_on_frontier=False,
                     curve_line_width=1.0, frontier_line_width=4.0,
                     if_y_axis_prob=False,
                     if_format_y_numbers=True,
                     legend_font_size=7,
                     frontier_label_shift_x=-0.01,
                     frontier_label_shift_y=0.01):

    for curve in curves:
        # plot line
        ax.plot(curve.xs, curve.ys * y_axis_multiplier,
                c=curve.color, alpha=transparency_lines,
                linewidth=curve_line_width, linestyle=curve.linestyle, label=curve.label)

        # plot intervals
        if curve.l_errs is not None and curve.u_errs is not None:
            ax.fill_between(curve.xs,
                            (curve.ys - curve.l_errs) * y_axis_multiplier,
                            (curve.ys + curve.u_errs) * y_axis_multiplier,
                            color=curve.color, alpha=transparency_intervals)
        # plot frontier
        if show_frontier:
            # check if this strategy is not dominated
            if curve.maxXs is not None and len(curve.maxXs) > 0:
                y = [y*y_axis_multiplier if y is not None else None for y in curve.maxYs]
                ax.plot(curve.maxXs, y,
                        c=curve.color, alpha=1, linewidth=frontier_line_width)

    if show_legend:
        ax.legend(loc=2, fontsize=legend_font_size)

    ax.set_title(title)
    ax.set_xlabel(x_label)
    ax.set_ylabel(y_label)
    ax.set_ylim(y_range)

    # add labels on the frontier
    if show_labels_on_frontier:
        y_min, y_max = ax.get_ylim()
        y_axis_length = y_max - y_min
        for curve in curves:
            if curve.maxXs is not None and len(curve.maxXs) > 0:
                if curve.maxYs[0] is not None and curve.maxYs[-1] is not None:
                    x_axis_length = x_values[-1] - x_values[0]
                    x = 0.5 * (curve.maxXs[0] + curve.maxXs[-1]) + frontier_label_shift_x * x_axis_length
                    y = 0.5 * (curve.maxYs[0] + curve.maxYs[-1]) * y_axis_multiplier \
                        + frontier_label_shift_y * y_axis_length
                    ax.text(x=x, y=y, s=curve.label, fontsize=legend_font_size+1, c=curve.color)

    # do the other formatting
    format_ax(ax=ax, y_range=y_range,
              x_range=[x_values[0], x_values[-1]], x_delta=x_delta,
              if_y_axis_prob=if_y_axis_prob,
              if_format_y_numbers=if_format_y_numbers,
              y_axis_decimal=y_axis_decimal)


import deampy.decision_tree as dt

# Migraine model displayed here: tests/DecisionTree/Migraine Model.tif

# dictionary for decision nodes
#               // key: cost, utility, [future nodes]
dictDecisions = {'d1': [0,     0,       ['c1', 'c5']]}

# dictionary for chance nodes
#           // key: cost,   utility,  [future nodes],  [probabilities]
dictChances = {'c1': [16.1,   0,       ['c2', 'c3'],    [.558, .442]],
               'c2': [0,      0,       ['t1', 't2'],    [.594, .406]],
               'c3': [0,      0,       ['t3', 'c4'],    [.92, .08]],
               'c4': [63.16,  0,       ['t4', 't5'],    [.998, .002]],
               'c5': [1.32,   0,       ['c6', 'c7'],    [.379, .621]],
               'c6': [0,      0,       ['t6', 't7'],    [.703, .297]],
               'c7': [0,      0,       ['t8', 'c8'],    [.92, .08]],
               'c8': [63.13,  0,       ['t9', 't10'],   [.998, .002]]
               }

# dictionary for terminal nodes
#               //key: cost, utility
dictTerminals = {'t1': [0,      1],
                 't2': [16.10,  .9],
                 't3': [0,     -.3],
                 't4': [0,      .1],
                 't5': [1093,  -.3],
                 't6': [0,       1],
                 't7': [1.32,   .9],
                 't8': [0,     -.3],
                 't9': [0,      .1],
                 't10': [1093, -.3]
                 }

# build the decision tree
myDT = dt.DecisionNode('d1', dictDecisions, dictChances, dictTerminals)
myDT.evaluate()

# print the expected cost and utility of each alternative
print('\nExpected cost and utility:')
print(myDT.get_cost_utility())

# print the probability of terminal nodes under each alternative
print('\nProbabilities of terminal nodes:')
print(myDT.get_terminal_prob())

# plot the expected cost and utility of each alternative
dt.graph_outcomes(myDT)



from setuptools import setup

setup(
    name='deampy',
    version='1.0.6',
    install_requires=['numpy', 'matplotlib', 'scipy'],
    packages=['deampy', 'deampy.optimization'],
    url='https://github.com/modeling-health-care-decisions/deampy',
    license='MIT License',
    author='Reza Yaesoubi',
    author_email='reza.yaesoubi@yale.edu',
    description='Decision analysis in medicine and public health'
)



import matplotlib.pyplot as plt

import deampy.plots.plot_support as Fig
from deampy.sample_path import *


def plot_sample_path(sample_path,
                     title=None, x_label=None, y_label=None,
                     figure_size=None, file_name=None,
                     legend=None, color_code=None, connect='step'):
    """
    plot a sample path
    :param sample_path: a sample path
    :param title: (string) title of the figure
    :param x_label: (string) x-axis label
    :param y_label: (string) y-axis label
    :param figure_size: (tuple) figure size
    :param file_name: (string) filename to to save the histogram as (e.g. 'fig.png')
    :param legend: (string) the legend
    :param color_code: (string) for example: 'b' blue 'g' green 'r' red 'c' cyan 'm' magenta 'y' yellow 'k' black
    :param connect: (string) set to 'step' to produce an step graph and to 'line' to produce a line graph
    """

    if not (isinstance(sample_path, IncidenceSamplePath) or isinstance(sample_path, PrevalenceSamplePath)):
        raise ValueError(
            'sample_path should be an instance of PrevalenceSamplePath or PrevalencePathBatchUpdate.')

    fig, ax = plt.subplots(figsize=figure_size)
    ax.set_title(title)  # title
    ax.set_xlabel(x_label)  # x-axis label
    ax.set_ylabel(y_label)  # y-axis label

    # add a sample path to this ax
    add_sample_path_to_ax(sample_path=sample_path,
                          ax=ax,
                          color_code=color_code,
                          legend=legend,
                          connect=connect)
    ax.set_ylim(bottom=0)  # the minimum has to be set after plotting the values

    if isinstance(sample_path, PrevalenceSamplePath):
        ax.set_xlim(left=0)
    elif isinstance(sample_path, IncidenceSamplePath):
        ax.set_xlim(left=0.5)

    # output figure
    Fig.output_figure(fig, file_name)


def plot_sample_paths(sample_paths,
                      title=None, x_label=None, y_label=None,
                      x_range=None, y_range=None,
                      figure_size=None, file_name=None,
                      legends=None, transparency=1,
                      color_codes=None,
                      common_color_code=None, connect='step'):
    """ graphs multiple sample paths
    :param sample_paths: a list of sample paths
    :param title: (string) title of the figure
    :param x_label: (string) x-axis label
    :param y_label: (string) y-axis label
    :param x_range: (list) [x_min, x_max]
    :param y_range: (list) [y_min, y_max]
    :param figure_size: (tuple) figure size
    :param file_name: (string) filename to to save the histogram as (e.g. 'fig.png')
    :param legends: (list) of strings for legend
    :param transparency: (float) 0.0 transparent through 1.0 opaque
    :param color_codes: (list) of color code for sample paths
    :param common_color_code: (string) color code if all sample paths should have the same color
        'b'	blue 'g' green 'r' red 'c' cyan 'm' magenta 'y' yellow 'k' black
    :param connect: (string) set to 'step' to produce an step graph and to 'line' to produce a line graph
    """

    if len(sample_paths) == 1:
        raise ValueError('Only one sample path is provided. Use graph_sample_path instead.')

    fig, ax = plt.subplots(figsize=figure_size)
    ax.set_title(title)  # title
    ax.set_xlabel(x_label)  # x-axis label
    ax.set_ylabel(y_label)  # y-axis label
    if x_range is not None:
        ax.set_xlim(x_range)
    if y_range is not None:
        ax.set_ylim(y_range)

    # add all sample paths
    add_sample_paths_to_ax(sample_paths=sample_paths,
                           ax=ax,
                           color_codes=color_codes,
                           common_color_code=common_color_code,
                           transparency=transparency,
                           connect=connect)

    # add legend if provided
    if legends is not None:
        if common_color_code is None:
            ax.legend(legends)
        else:
            ax.legend([legends])

    # set the minimum of y-axis to zero
    ax.set_ylim(bottom=0)  # the minimum has to be set after plotting the values
    # output figure
    Fig.output_figure(fig, file_name)


def plot_sets_of_sample_paths(sets_of_sample_paths,
                              title=None, x_label=None, y_label=None,
                              x_range=None, y_range=None,
                              figure_size=None, file_name=None,
                              legends=None, transparency=1, color_codes=None, connect='step'):
    """ graphs multiple sample paths
    :param sets_of_sample_paths: (list) of list of sample paths
    :param title: (string) title of the figure
    :param x_label: (string) x-axis label
    :param y_label: (string) y-axis label
    :param x_range: (list) [x_min, x_max]
    :param y_range: (list) [y_min, y_max]
    :param figure_size: (tuple) figure size
    :param file_name: (string) filename to to save the histogram as (e.g. 'fig.png')
    :param legends: (list of strings) for legends
    :param transparency: float (0.0 transparent through 1.0 opaque)
    :param color_codes: (list of strings) color code of sample path sets
            e.g. 'b' blue 'g' green 'r' red 'c' cyan 'm' magenta 'y' yellow 'k' black
    :param connect: (string) set to 'step' to produce an step graph and to 'line' to produce a line graph
    """

    if len(sets_of_sample_paths) == 1:
        raise ValueError('Only one set of sample paths is provided. Use plot_sample_paths instead.')

    fig, ax = plt.subplots(figsize=figure_size)
    ax.set_title(title)  # title
    ax.set_xlabel(x_label)  # x-axis label
    ax.set_ylabel(y_label)  # y-axis label
    if x_range is not None:
        ax.set_xlim(x_range)
    if y_range is not None:
        ax.set_ylim(y_range)

    # add all sample paths
    add_sets_of_sample_paths_to_ax(sets_of_sample_paths=sets_of_sample_paths,
                                   ax=ax,
                                   color_codes=color_codes,
                                   legends=legends,
                                   transparency=transparency,
                                   connect=connect)

    # set the minimum of y-axis to zero
    ax.set_ylim(bottom=0)  # the minimum has to be set after plotting the values
    # output figure
    Fig.output_figure(fig, file_name)


def add_sample_path_to_ax(sample_path, ax, color_code=None, legend=None, transparency=1, connect='step'):

    # x and y values
    if isinstance(sample_path, PrevalenceSamplePath):
        x_values = sample_path.get_times()
    elif isinstance(sample_path, IncidenceSamplePath):
        x_values = sample_path.get_period_numbers()

    y_values = sample_path.get_values()

    # plot the sample path
    if connect == 'step':
        ax.step(x=x_values, y=y_values, where='post', color=color_code,
                linewidth=0.75, label=legend, alpha=transparency)
    else:
        ax.plot(x_values, y_values, color=color_code,
                linewidth=0.75, label=legend, alpha=transparency)

    # add legend if provided
    if legend is not None:
        ax.legend()


def add_sample_paths_to_ax(sample_paths, ax, color_codes, common_color_code, transparency, connect):

    # add every path
    for i, path in enumerate(sample_paths):
        if color_codes is not None:
            color = color_codes[i]
        elif common_color_code is not None:
            color = common_color_code
        else:
            color = None
        add_sample_path_to_ax(sample_path=path,
                              ax=ax,
                              color_code=color,
                              transparency=transparency,
                              connect=connect)


def add_sets_of_sample_paths_to_ax(sets_of_sample_paths, ax, color_codes, legends, transparency, connect):

    # add every path
    for i, sample_paths in enumerate(sets_of_sample_paths):
        for j, path in enumerate(sample_paths):
            if j == 0:
                legend = legends[i]
            else:
                legend = None
            if color_codes is None:
                this_color_code = None
            else:
                this_color_code = color_codes[i]
            add_sample_path_to_ax(sample_path=path,
                                  ax=ax,
                                  color_code=this_color_code,
                                  legend=legend,
                                  transparency=transparency,
                                  connect=connect)


from deampy.optimization.appx_policy_itr import GreedyApproxDecisionMaker
from model import *

N = 1000
ACTION_COST = 50
COST_SIGMA = 0
Q_FUNC_DEGREE = 2
Q_FUNC_CSVFILE = 'q-functions.csv' # or optimal_q_functions.csv


def compare(q_function_degree=None, q_functions_csvfile=None):

    if q_function_degree is None:
        q_function_degree = Q_FUNC_DEGREE
    if q_functions_csvfile is None:
        q_functions_csvfile = Q_FUNC_CSVFILE

    # always-off strategy
    multi_model = MultiModel(decision_rule=AlwaysOff(),
                             cost_sigma=COST_SIGMA, action_cost=ACTION_COST)
    multi_model.simulate(n=N)
    print('Always off: ', multi_model.statCost.get_formatted_mean_and_interval(
        interval_type='c', sig_digits=4))

    # always-on strategy
    multi_model = MultiModel(decision_rule=AlwaysOn(),
                             cost_sigma=COST_SIGMA, action_cost=ACTION_COST)
    multi_model.simulate(n=N)
    print('Always on: ', multi_model.statCost.get_formatted_mean_and_interval(
        interval_type='c', sig_digits=4))

    # myopic strategy
    multi_model = MultiModel(decision_rule=Myopic(),
                             cost_sigma=COST_SIGMA, action_cost=ACTION_COST)
    multi_model.simulate(n=N)
    print('Myopic: ', multi_model.statCost.get_formatted_mean_and_interval(
        interval_type='c', sig_digits=4))

    # dynamic strategy
    approx_decision_maker = GreedyApproxDecisionMaker(num_of_actions=1,
                                                      q_function_degree=q_function_degree,
                                                      q_functions_csv_file=q_functions_csvfile)
    multi_model = MultiModel(decision_rule=Dynamic(approx_decision_maker=approx_decision_maker),
                             cost_sigma=COST_SIGMA, action_cost=ACTION_COST)
    multi_model.simulate(n=N)
    print('Dynamic: ', multi_model.statCost.get_formatted_mean_and_interval(
        interval_type='c', sig_digits=4))


if __name__ == '__main__':
    compare()



from deampy.support.econ_eval_support import *


w = find_intersecting_wtp(w0=0,
                          u_new=inmb_u(d_cost=5000, d_effect=0),
                          u_base=inmb_u(d_cost=0, d_effect=0))
print(w)


import matplotlib.pyplot as plt
import matplotlib.patches as patches

fig = plt.figure()
ax = fig.add_subplot(111, aspect='equal')

# Parallelogram

x = 0.5
width = 0.1
mean = 0.5
error = 0.1
xy = [
    [x, mean-error],
    [x-width, mean],
    [x, mean+error],
    [x+width, mean]
]

x = [0.3,0.6,.7,.4]
y = [0.4,0.4,0.6,0.6]
ax.add_patch(patches.Polygon(xy=xy, fill=False))


plt.show()


import random

import numpy as np


def proper_file_name(text):
    """
    :param text: filename
    :return: filename where invalid characters are removed
    """

    return text.replace('|', ',').replace('<', 'l').replace('>', 'g').replace('\n', '')


def get_moving_average(data, window=2):
    """
    calculates the moving average of a time-series
    :param data: list of observations
    :param window: the window (number of data points) over which the average should be calculated
    :return: list of moving averages
    """

    if window < 2:
        raise ValueError('The window over which the moving averages '
                         'should be calculated should be greater than 1.')
    if window >= len(data):
        raise ValueError('The window over which the moving averages '
                         'should be calculated should be less than the number of data points.')

    window = int(window)

    averages = []

    # the 'window - 1' averages cannot be calculated
    for i in range(window-1):
        averages.append(None)

    # calculate the first moving average
    total = 0
    n = 0
    for i in range(window):
        if data[i] is not None:
            total += data[i]
            n += 1
    moving_ave = total/n
    averages.append(moving_ave)

    for i in range(window, len(data)):
        if moving_ave is None:
            moving_sum = 0
        else:
            moving_sum = moving_ave * n
        if data[i-window] is not None:
            moving_sum -= data[i-window]
            n -= 1
        if data[i] is not None:
            moving_sum += data[i]
            n += 1

        if n > 0:
            moving_ave = moving_sum/n
        else:
            moving_ave = None
        averages.append(moving_ave)

    return averages


def effective_sample_size(likelihood_weights):
    """
    :param likelihood_weights: (list) probabilities
    :returns: the effective sample size
    """

    # convert to numpy array if needed
    if not type(likelihood_weights) == np.ndarray:
        likelihood_weights = np.array(likelihood_weights)

    # normalize the weights if needed
    s = sum(likelihood_weights)
    if s < 0.99999 or s > 1.00001:
        likelihood_weights = likelihood_weights / s

    return 1 / np.sum(likelihood_weights ** 2)


def get_random_colors(n, seed=0):
    """
    :param n: (int) number of colors randomly selected 
    :param seed: (int) seed of the random number generator 
    :return: (list) of color codes 
    """
    
    random.seed(seed)

    colors = []
    for j in range(n):
        rand_colors = "#" + ''.join([random.choice('ABCDEF0123456789') for i in range(6)])
        colors.append(rand_colors)
    
    return colors


def convert_lnl_to_prob(ln_likelihoods):

    for i, lnl in enumerate(ln_likelihoods):
        if np.isnan(lnl):
            ln_likelihoods[i] = -np.inf

    # find the maximum lnl
    max_lnl = max(ln_likelihoods)

    # find probability weights
    probs = [np.exp(s - max_lnl) for s in ln_likelihoods]
    sum_prob = sum(probs)

    # normalize the probability weights
    return np.array(probs)/sum_prob


def get_percentile_of_empirical_dist(xs, probs, q):
    """
    :param xs: (list or np.array) values that the random variable can take
        (assumed to be sorted in the increasing order)
    :param probs: (list or np.array) probability of each value
    :param q: (float) percentile value (has to be between 0 and 1)
    :return: the value of x where q*100% of cumulative probability distribution is below q
    """

    raise ValueError('needs to be debugged.')

    if not 0 <= q <= 1:
        raise ValueError('q should be between 0 and 1.')

    # remove x values with 0 probability
    xs_nonzero_prob = []
    nonzero_probs = []
    for i, p in enumerate(probs):
        if p > 0:
            xs_nonzero_prob.append(xs[i])
            nonzero_probs.append(p)

    sum_p = 0

    if q <= 0.5:
        i = 0
        while True:
            sum_p += nonzero_probs[i]
            if sum_p >= q:
                if i - 1 < 0:
                    return None
                else:
                    return xs_nonzero_prob[i - 1]
            i += 1

    else:
        i = len(xs_nonzero_prob) - 1
        while True:
            if q == 1:
                return xs_nonzero_prob[-1]
            else:
                sum_p += nonzero_probs[i]
                if sum_p >= 1 - q:
                    return xs_nonzero_prob[i - 1]
                i -= 1

#
# print(get_percentile_of_empirical_dist(xs=[1, 2, 3, 4], probs=[0.1, 0.2, 0.7, 0], q=0.05))
# print(get_percentile_of_empirical_dist(xs=[1, 2, 3, 4], probs=[0.1, 0.2, 0.7, 0], q=0.4))
# print(get_percentile_of_empirical_dist(xs=[1, 2, 3, 4], probs=[0.1, 0.2, 0.7, 0], q=1))
# print(get_percentile_of_empirical_dist(xs=[1, 2, 3, 4], probs=[0.1, 0.2, 0.7, 0], q=0.6))





def takeSecond(elem):
    return elem[1]


list = [
    [1, 2],
    [2, 4],
    [3, 3],
]

print(list)
list.sort(key=takeSecond)

print(list)
subset = [row for row in list if row[0]==2]

print(subset)
subset[0][1] = 23

print(subset)
print(list)


import numpy as np

import deampy.plots.prob_dists as Plot
import deampy.random_variats as RVGs
from tests.prob_dists.rvg_tests import get_samples


def test_fitting_beta():

    print("\nTesting Beta with a=2, b=3, loc=1, scale=2:")
    dist = RVGs.Beta(a=2, b=3, loc=1, scale=2)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    # method of moment
    dict_mm_results = RVGs.Beta.fit_mm(
        mean=np.mean(data), st_dev=np.std(data), minimum=1, maximum=3)
    # maximum likelihood
    dict_ml_results = RVGs.Beta.fit_ml(data=data, minimum=1, maximum=3)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_beta_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_beta_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_beta_binomial():

    print("\nTesting Beta-Binomial with n=100, a=2, b=3, loc=1, scale=2:")
    dist = RVGs.BetaBinomial(n=20, a=2, b=3, loc=1)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    # method of moment
    dict_mm_results = RVGs.BetaBinomial.fit_mm(
        mean=np.mean(data), st_dev=np.std(data), n=20, fixed_location=1)
    # maximum likelihood
    dict_ml_results = RVGs.BetaBinomial.fit_ml(data=data, fixed_location=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_beta_binomial_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_beta_binomial_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_binomial():

    print("\nTesting Binomial with n=100, p=0.3, loc=1:")
    dist = RVGs.Binomial(n=100, p=0.3, loc=1)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    # method of moment
    dict_mm_results = RVGs.Binomial.fit_mm(
        mean=np.mean(data), st_dev=np.std(data), fixed_location=1)
    # maximum likelihood
    dict_ml_results = RVGs.Binomial.fit_ml(
        data=data, fixed_location=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_binomial_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_binomial_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_empirical():

    print("\nTesting empirical with p=[0.1, 0.2, 0.7]")
    dist = RVGs.Empirical(probabilities=[0.1, 0.2, 0.7])

    data = np.array(get_samples(dist, np.random))
    # method of moments
    dict_mm_results = RVGs.Empirical.fit_mm(
        data=data, bin_size=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)


def test_fitting_exponential():

    print("\nTesting Exponential with scale=0.5, loc=2")
    dist = RVGs.Exponential(scale=0.5, loc=2)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    # method of moment
    dict_mm_results = RVGs.Exponential.fit_mm(
        mean=np.average(data), fixed_location=2)
    # maximum likelihood
    dict_ml_results = RVGs.Exponential.fit_ml(
        data=data, fixed_location=2)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_exponential_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_exponential_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_gamma():

    print('\nTesting Gamma with a=10, scale=1, loc=2:')
    dist = RVGs.Gamma(a=10, scale=1, loc=2)
    print('  mean, st dev: ', dist.get_mean_st_dev())
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.Gamma.fit_mm(
        mean=np.average(data), st_dev=np.std(data), fixed_location=2)
    dict_ml_results = RVGs.Gamma.fit_ml(
        data=data, fixed_location=2)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_gamma_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_gamma_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_gamma_poisson():

    print("\nTesting Gamma Poisson with a=2, gamma_scale=4, loc=2")
    dist = RVGs.GammaPoisson(a=2, gamma_scale=4, loc=2)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.GammaPoisson.fit_mm(
        mean=np.average(data), st_dev=np.std(data), fixed_location=2)
    dict_ml_results = RVGs.GammaPoisson.fit_ml(
        data=data, fixed_location=2)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_gamma_poisson_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_gamma_poisson_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_geometric():
    print("\nTesting Geometric with p=0.3, loc=1")
    dist = RVGs.Geometric(p=0.3, loc=1)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.Geometric.fit_mm(
        mean=np.average(data), fixed_location=1)
    dict_ml_results = RVGs.Geometric.fit_ml(
        data=data, fixed_location=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_geometric_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_geometric_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_johnson_sb():
    print("\nTesting Johnson Sb with a=10, b=5, loc=10, scale=100")
    dist = RVGs.JohnsonSb(a=10, b=5, loc=10, scale=100)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_ml_results = RVGs.JohnsonSb.fit_ml(
        data=data, fixed_location=10)

    print("  Fit:")
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_johnson_sb_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_johnson_su():

    print("\nTesting Johnson Su with a=2, b=3, loc=1, scale=4")
    dist = RVGs.JohnsonSu(a=2, b=3, loc=1, scale=4)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_ml_results = RVGs.JohnsonSu.fit_ml(
        data=data, fixed_location=1)

    print("  Fit:")
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_johnson_su_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_lognormal():

    print("\nTesting LogNormal with mu=0.2, sigma=0.1, loc=1")
    dist = RVGs.LogNormal(mu=0.2, sigma=0.1, loc=1)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.LogNormal.fit_mm(
        mean=np.average(data), st_dev=np.std(data), fixed_location=1)
    dict_ml_results = RVGs.LogNormal.fit_ml(
        data=data, fixed_location=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_lognormal_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_lognormal_fit(data=data, fit_results=dict_mm_results, title='Maximum Likelihood')


def test_fitting_normal():

    print("\nTesting Normal with loc=10, scale=2")
    dist = RVGs.Normal(loc=10, scale=2)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.Normal.fit_mm(
        mean=np.average(data), st_dev=np.std(data))
    dict_ml_results = RVGs.Normal.fit_ml(
        data=data)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_normal_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_normal_fit(data=data, fit_results=dict_mm_results, title='Maximum Likelihood')


def test_fitting_negbinomial():
    print("\nTesting NegBinomial with n=10, p=0.2, loc=1")
    dist = RVGs.NegativeBinomial(n=10, p=0.2, loc=1)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.NegativeBinomial.fit_mm(
        mean=np.average(data), st_dev=np.std(data), fixed_location=1)
    dict_ml_results = RVGs.NegativeBinomial.fit_ml(
        data=data, fixed_location=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_negbinomial_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    # Plot.plot_negbinomial_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_poisson():

    print("\nTesting Poisson with mean=100 and loc = 10")
    dist = RVGs.Poisson(mu=100, loc=10)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.Poisson.fit_mm(mean=np.average(data), fixed_location=10)
    dict_ml_results = RVGs.Poisson.fit_ml(data=data, fixed_location=10)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_poisson_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_poisson_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_triangular():

    print("\nTesting triangular with c=0.2, loc=6, scale=7")
    dist = RVGs.Triangular(c=0.2, loc=6, scale=7)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_ml_results = RVGs.Triangular.fit_ml(data=data, fixed_location=6)

    print("  Fit:")
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_triangular_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_uniform():

    print("\nTesting triangular with scale=10, loc=2")
    dist = RVGs.Uniform(scale=10, loc=2)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.Uniform.fit_mm(mean=np.average(data), st_dev=np.std(data))
    dict_ml_results = RVGs.Uniform.fit_ml(data=data)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_uniform_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_uniform_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_uniform_discrete():

    print("\nTesting uniform discrete with l=10, u=18")
    dist = RVGs.UniformDiscrete(l=10, u=18)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.UniformDiscrete.fit_mm(mean=np.average(data), st_dev=np.std(data))
    dict_ml_results = RVGs.UniformDiscrete.fit_ml(data=data)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_uniform_discrete_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_uniform_discrete_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')


def test_fitting_weibull():

    print("\nTesting Weibull with a=5, scale=2, loc=1")
    dist = RVGs.Weibull(a=5, scale=2, loc=1)
    print('  percentile interval: ', dist.get_percentile_interval(alpha=0.05))

    data = np.array(get_samples(dist, np.random))
    dict_mm_results = RVGs.Weibull.fit_mm(
        mean=np.average(data), st_dev=np.std(data), fixed_location=1)
    dict_ml_results = RVGs.Weibull.fit_ml(
        data=data, fixed_location=1)

    print("  Fit:")
    print("    MM:", dict_mm_results)
    print("    ML:", dict_ml_results)

    # plot the fitted distributions
    Plot.plot_weibull_fit(data=data, fit_results=dict_mm_results, title='Method of Moment')
    Plot.plot_weibull_fit(data=data, fit_results=dict_ml_results, title='Maximum Likelihood')




import numpy as np

from deampy.random_variats import Empirical, Exponential


class MarkovJumpProcess:

    def __init__(self, transition_prob_matrix):
        """
        :param transition_prob_matrix: (list) transition probability matrix of a discrete-time Markov model
        """

        assert type(transition_prob_matrix) is list, \
            'Transition probability matrix should be a list'

        self._empiricalDists = []

        for i, probs in enumerate(transition_prob_matrix):

            # check if the sum of probabilities in this row is 1.
            s = sum(probs)
            if s < 0.99999 or s > 1.00001:
                raise ValueError('Sum of each row in a probability matrix should be 1. '
                                 'Sum of row {0} is {1}.'.format(i, s))

            # create an empirical distribution over the future states from this state
            self._empiricalDists.append(Empirical(probabilities=probs))

    def get_next_state(self, current_state_index, rng):
        """
        :param current_state_index: index of the current state
        :param rng: random number generator object
        :return: the index of the next state
        """

        if not (0 <= current_state_index < len(self._empiricalDists)):
            raise ValueError('The value of the current state index should be greater '
                             'than 0 and smaller than the number of states.'
                             'Value provided for current state index is {}.'.format(current_state_index))

        # find the next state index by drawing a sample from
        # the empirical distribution associated with this state
        return self._empiricalDists[current_state_index].sample(rng=rng)


class Gillespie:
    def __init__(self, transition_rate_matrix):
        """
        :param transition_rate_matrix: transition rate matrix of the continuous-time Markov model
        """

        assert isinstance(transition_rate_matrix, list), \
            'transition_rate_matrix should be an array, {} was provided'.format(type(transition_rate_matrix))

        if len(transition_rate_matrix) == 0:
            raise ValueError('An empty transition_rate_matrix is provided.')

        self._rateMatrix = transition_rate_matrix
        self._expDists = []
        self._empiricalDists = []

        for i, row in enumerate(transition_rate_matrix):
            # find sum of rates out of this state
            rate_out = out_rate(row, i)
            # if the rate is 0, put None as the exponential and empirical distributions
            if rate_out > 0:
                # create an exponential distribution with rate equal to sum of rates out of this state
                self._expDists.append(Exponential(scale=1/rate_out))
                # find the transition rates to other states
                # assume that the rate into this state is 0
                rates = []
                for j, v in enumerate(row):
                    if i == j:
                        rates.append(0)
                    else:
                        rates.append(v)

                # calculate the probability of each event (prob_j = rate_j / (sum over j of rate_j)
                probs = np.array(rates) / rate_out
                # create an empirical distribution over the future states from this state
                self._empiricalDists.append(Empirical(probs))

            else:  # if the sum of rates out of this state is 0
                self._expDists.append(None)
                self._empiricalDists.append(None)

    def get_next_state(self, current_state_index, rng):
        """
        :param current_state_index: index of the current state
        :param rng: random number generator object
        :return: (dt, i) where dt is the time until next event, and i is the index of the next state
         it returns None for dt if the process is in an absorbing state
        """

        if not (0 <= current_state_index < len(self._rateMatrix)):
            raise ValueError('The value of the current state index should be greater '
                             'than 0 and smaller than the number of states.')

        # if this is an absorbing state (i.e. sum of rates out of this state is 0)
        if self._expDists[current_state_index] is None:
            # the process stays in the current state
            dt = None
            i = current_state_index
        else:
            # find the time until next event
            dt = self._expDists[current_state_index].sample(rng=rng)
            # find the next state
            i = self._empiricalDists[current_state_index].sample(rng=rng)

        return dt, i


def continuous_to_discrete(trans_rate_matrix, delta_t):
    """
    :param trans_rate_matrix: (list of lists) transition rate matrix (assumes None or 0 for diagonal elements)
    :param delta_t: cycle length
    :return: transition probability matrix (list of lists)
             and the upper bound for the probability of two transitions within delta_t (float)
        converting [p_ij] to [lambda_ij] where
            mu_i = sum of rates out of state i
            p_ij = exp(-mu_i*delta_t),      if i = j,
            p_ij = (1-exp(-mu_i*delta_t))*lambda_ij/mu_i,      if i != j.

    """

    # list of rates out of each row
    rates_out = []
    for i, row in enumerate(trans_rate_matrix):
        rates_out.append(out_rate(row, i))

    prob_matrix = []
    for i in range(len(trans_rate_matrix)):
        prob_row = []   # list of probabilities
        # calculate probabilities
        for j in range(len(trans_rate_matrix[i])):
            prob = 0
            if i == j:
                prob = np.exp(-rates_out[i] * delta_t)
            else:
                if rates_out[i] > 0:
                    prob = (1 - np.exp(-rates_out[i] * delta_t)) * trans_rate_matrix[i][j] / rates_out[i]
            # append this probability
            prob_row.append(prob)

        # append this row of probabilities
        prob_matrix.append(prob_row)

    # probability that transition occurs within delta_t for each state
    probs_out = []
    for rate in rates_out:
        probs_out.append(1-np.exp(-delta_t*rate))

    # calculate the probability of two transitions within delta_t for each state
    prob_out_out = []
    for i in range(len(trans_rate_matrix)):

        # probability of leaving state i withing delta_t
        prob_out_i = probs_out[i]

        # probability of leaving the new state after i withing delta_t
        prob_out_again = 0

        for j in range(len(trans_rate_matrix[i])):
            if not i == j:

                # probability of transition from i to j
                prob_i_j = 0
                if rates_out[i]>0:
                    prob_i_j = trans_rate_matrix[i][j] / rates_out[i]
                # probability of transition from i to j and then out of j within delta_t
                prob_i_j_out = prob_i_j * probs_out[j]
                # update the probability of transition out of i and again leaving the new state
                prob_out_again += prob_i_j_out

        # store the probability of leaving state i to a new state and leaving the new state withing delta_t
        prob_out_out.append(prob_out_i*prob_out_again)

    # return the probability matrix and the upper bound for the probability of two transitions with delta_t
    return prob_matrix, max(prob_out_out)


def out_rate(rates, idx):
    """
    :param rates: list of rates leaving this state
    :param inx: index of this state
    :returns the rate of leaving this sate (the sum of rates)
    """

    sum_rates = 0
    for i, v in enumerate(rates):
        if i != idx and v is not None:
            sum_rates += v
    return sum_rates


def discrete_to_continuous(trans_prob_matrix, delta_t):
    """
    :param trans_prob_matrix: (list of lists) transition probability matrix
    :param delta_t: cycle length
    :return: (list of lists) transition rate matrix
        Converting [p_ij] to [lambda_ij] where
            lambda_ii = None, and
            lambda_ij = -ln(p_ii) * p_ij / ((1-p_ii)*Delta_t)
    """

    assert type(trans_prob_matrix) == list, \
        "prob_matrix is a matrix that should be represented as a list of lists: " \
        "For example: [ [0.1, 0.9], [0.8, 0.2] ]."

    rate_matrix = []
    for i, row in enumerate(trans_prob_matrix):
        rate_row = []   # list of rates
        # calculate rates
        for j in range(len(row)):
            # rate is None for diagonal elements
            if i == j:
                rate = None
            else:
                # rate is zero if this is an absorbing state
                if trans_prob_matrix[i][i] == 1:
                    rate = 0
                else:
                    rate = -np.log(trans_prob_matrix[i][i]) * trans_prob_matrix[i][j] / ((1 - trans_prob_matrix[i][i]) * delta_t)
            # append this rate
            rate_row.append(rate) 
        # append this row of rates
        rate_matrix.append(rate_row)

    return rate_matrix



from tests.prob_dists.fit_dist_tests import *

# simulate some data
np.random.seed(1)
test_fitting_beta()
test_fitting_beta_binomial()
test_fitting_binomial()
test_fitting_empirical()
test_fitting_exponential()
test_fitting_gamma()
test_fitting_gamma_poisson()
test_fitting_geometric()
test_fitting_johnson_sb()
test_fitting_johnson_su()
test_fitting_lognormal()
test_fitting_negbinomial()
test_fitting_normal()
test_fitting_poisson()
test_fitting_triangular()
test_fitting_uniform()
test_fitting_uniform_discrete()
test_fitting_weibull()


